@@ -753,7 +753,8 @@ func prepareHAEnvironment(t *testing.T, db *pg.DB) (loadBalancing *dbmodel.Servi
 
 	// Detect HA services
 	for _, daemon := range daemons {
-		services := DetectHAServices(db, daemon)
+		services, err := DetectHAServices(db, daemon)
+		require.NoError(t, err)
 		err = dbmodel.CommitServicesIntoDB(db, services, daemon)
 		require.NoError(t, err)
 	}
@@ -905,7 +906,9 @@ func TestGetHATestConfigWithSubnets(t *testing.T) {
 	path, params, ok := config.GetHookLibraries().GetHAHookLibrary()
 	require.True(t, ok)
 	require.NotEmpty(t, path)
-	require.Equal(t, "server1", *params.GetFirst().ThisServerName)
+	relationships := params.GetAllRelationships()
+	require.Len(t, relationships, 1)
+	require.Equal(t, "server1", *relationships[0].ThisServerName)
 	subnets := config.GetSubnets()
 	require.NotEmpty(t, subnets)
 }
