@@ -7,6 +7,7 @@ import (
 
 	"github.com/go-pg/pg/v10"
 	"github.com/stretchr/testify/require"
+	keaconfig "isc.org/stork/appcfg/kea"
 	dhcpmodel "isc.org/stork/datamodel/dhcp"
 	dbtest "isc.org/stork/server/database/test"
 	storkutil "isc.org/stork/util"
@@ -130,6 +131,30 @@ func addTestHosts(t *testing.T, db *pg.DB) []Host {
 	return hosts
 }
 
+// Test that the function returns true if the data source is config; otherwise,
+// it should return false.
+func TestHostDataSourceIsConfig(t *testing.T) {
+	require.True(t, HostDataSourceConfig.IsConfig())
+	require.False(t, HostDataSourceAPI.IsConfig())
+	require.False(t, HostDataSourceUnspecified.IsConfig())
+}
+
+// Test that the function returns true if the data source is API; otherwise,
+// it should return false.
+func TestHostDataSourceIsAPI(t *testing.T) {
+	require.False(t, HostDataSourceConfig.IsAPI())
+	require.True(t, HostDataSourceAPI.IsAPI())
+	require.False(t, HostDataSourceUnspecified.IsAPI())
+}
+
+// Test that the function returns true if the data source is unspecified;
+// otherwise, it should return false.
+func TestHostDataSourceIsUnspecified(t *testing.T) {
+	require.False(t, HostDataSourceConfig.IsUnspecified())
+	require.False(t, HostDataSourceAPI.IsUnspecified())
+	require.True(t, HostDataSourceUnspecified.IsUnspecified())
+}
+
 // This test verifies that the new host along with identifiers and reservations
 // can be added to the database.
 func TestAddHost(t *testing.T) {
@@ -795,9 +820,8 @@ func TestGetHostsByPageGlobal(t *testing.T) {
 	hosts := addTestHosts(t, db)
 
 	// find only global hosts
-	global := true
 	filters := HostsByPageFilters{
-		Global: &global,
+		Global: storkutil.Ptr(true),
 	}
 	returned, total, err := GetHostsByPage(db, 0, 10, filters, "", SortDirAny)
 	require.NoError(t, err)
@@ -807,9 +831,8 @@ func TestGetHostsByPageGlobal(t *testing.T) {
 	require.Contains(t, returned, hosts[3])
 
 	// find only non-global hosts
-	global = false
 	filters = HostsByPageFilters{
-		Global: &global,
+		Global: storkutil.Ptr(false),
 	}
 	returned, total, err = GetHostsByPage(db, 0, 10, filters, "", SortDirAny)
 	require.NoError(t, err)
@@ -819,6 +842,466 @@ func TestGetHostsByPageGlobal(t *testing.T) {
 	require.Contains(t, []int64{hosts[0].ID, hosts[2].ID}, returned[1].ID)
 }
 
+// Test that the global hosts from a specific app can be fetched if both the
+// app ID and the global filter are set.
+func TestGetHostsByPageGlobalAndAppID(t *testing.T) {
+	// Arrange
+	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	// Add four hosts. Two global and two non-global.
+	hosts := addTestHosts(t, db)
+	daemon1, daemon2, _ := addTestDaemons(db)
+
+	_ = AddDaemonToHost(db, &hosts[0], daemon1.ID, HostDataSourceAPI)
+	_ = AddDaemonToHost(db, &hosts[1], daemon1.ID, HostDataSourceConfig)
+	_ = AddDaemonToHost(db, &hosts[2], daemon2.ID, HostDataSourceAPI)
+	_ = AddDaemonToHost(db, &hosts[3], daemon2.ID, HostDataSourceConfig)
+
+	// Prepare a filter.
+	filters := HostsByPageFilters{
+		Global: storkutil.Ptr(true),
+		AppID:  storkutil.Ptr(daemon1.AppID),
+	}
+
+	// Act
+	returned, total, err := GetHostsByPage(db, 0, 10, filters, "", SortDirAny)
+
+	// Assert
+	require.NoError(t, err)
+	require.EqualValues(t, 1, total)
+	require.Len(t, returned, 1)
+	require.EqualValues(t, hosts[1].ID, returned[0].ID)
+	require.Zero(t, returned[0].SubnetID)
+}
+
+// Test that the non-global hosts from a specific app can be fetched if both the
+// app ID and the negated global filter are set.
+func TestGetHostsByPageNotGlobalAndAppID(t *testing.T) {
+	// Arrange
+	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	// Add four hosts. Two global and two non-global.
+	hosts := addTestHosts(t, db)
+	daemon1, daemon2, _ := addTestDaemons(db)
+
+	_ = AddDaemonToHost(db, &hosts[0], daemon1.ID, HostDataSourceAPI)
+	_ = AddDaemonToHost(db, &hosts[1], daemon1.ID, HostDataSourceConfig)
+	_ = AddDaemonToHost(db, &hosts[2], daemon2.ID, HostDataSourceAPI)
+	_ = AddDaemonToHost(db, &hosts[3], daemon2.ID, HostDataSourceConfig)
+
+	// Prepare a filter.
+	filters := HostsByPageFilters{
+		Global: storkutil.Ptr(false),
+		AppID:  storkutil.Ptr(daemon1.AppID),
+	}
+
+	// Act
+	returned, total, err := GetHostsByPage(db, 0, 10, filters, "", SortDirAny)
+
+	// Assert
+	require.NoError(t, err)
+	require.EqualValues(t, 1, total)
+	require.Len(t, returned, 1)
+	require.EqualValues(t, hosts[0].ID, returned[0].ID)
+	require.NotZero(t, returned[0].SubnetID)
+}
+
+// Test that page of the hosts is empty if there is a filter for conflicted or
+// duplicated DHCP data and there are no such hosts.
+func TestGetHostsByPageNoConflictsOrDuplicates(t *testing.T) {
+	// Arrange
+	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	_ = addTestHosts(t, db)
+
+	filters := HostsByPageFilters{
+		DHCPDataConflict:  storkutil.Ptr(true),
+		DHCPDataDuplicate: storkutil.Ptr(true),
+	}
+
+	// Act
+	returned, total, err := GetHostsByPage(db, 0, 10, filters, "", SortDirAny)
+
+	// Assert
+	require.NoError(t, err)
+	require.Zero(t, total)
+	require.Empty(t, returned)
+}
+
+// Test that page of the hosts contains only hosts with duplicated local hosts
+// if the duplicated DHCP data filter is set.
+func TestGetHostsByPageDuplicate(t *testing.T) {
+	// Arrange
+	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	filters := HostsByPageFilters{
+		DHCPDataDuplicate: storkutil.Ptr(true),
+	}
+
+	hosts := addTestHosts(t, db)
+	host := hosts[0]
+	daemon, _, _ := addTestDaemons(db)
+
+	_ = AddDaemonToHost(db, &host, daemon.ID, HostDataSourceAPI)
+	_ = AddDaemonToHost(db, &host, daemon.ID, HostDataSourceConfig)
+
+	// Act
+	returned, total, err := GetHostsByPage(db, 0, 10, filters, "", SortDirAny)
+
+	// Assert
+	require.NoError(t, err)
+	require.EqualValues(t, 1, total)
+	require.Len(t, returned, 1)
+	require.EqualValues(t, host.ID, returned[0].ID)
+}
+
+// Test that page of the hosts contains only hosts with conflicted local hosts
+// if the conflicted DHCP data filter is set.
+func TestGetHostsByPageConflict(t *testing.T) {
+	// Arrange
+	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	filters := HostsByPageFilters{
+		DHCPDataConflict: storkutil.Ptr(true),
+	}
+
+	hosts := addTestHosts(t, db)
+	host := hosts[0]
+	daemon, _, _ := addTestDaemons(db)
+
+	hasher := keaconfig.NewHasher()
+	testCases := map[string][]LocalHost{
+		"DHCP options": {
+			{
+				HostID:     host.ID,
+				DaemonID:   daemon.ID,
+				DataSource: HostDataSourceAPI,
+				DHCPOptionSet: NewDHCPOptionSet([]DHCPOption{{
+					Code: 42,
+				}}, hasher),
+			},
+			{
+				HostID:     host.ID,
+				DaemonID:   daemon.ID,
+				DataSource: HostDataSourceConfig,
+				DHCPOptionSet: NewDHCPOptionSet([]DHCPOption{{
+					Code: 24,
+				}}, hasher),
+			},
+		},
+		"Client classes": {
+			{
+				HostID:        host.ID,
+				DaemonID:      daemon.ID,
+				DataSource:    HostDataSourceAPI,
+				ClientClasses: []string{"foo", "bar"},
+			},
+			{
+				HostID:        host.ID,
+				DaemonID:      daemon.ID,
+				DataSource:    HostDataSourceConfig,
+				ClientClasses: []string{"foo", "baz"},
+			},
+		},
+		"Boot filename": {
+			{
+				HostID:       host.ID,
+				DaemonID:     daemon.ID,
+				DataSource:   HostDataSourceAPI,
+				BootFileName: "foo",
+			},
+			{
+				HostID:       host.ID,
+				DaemonID:     daemon.ID,
+				DataSource:   HostDataSourceConfig,
+				BootFileName: "bar",
+			},
+		},
+		"Next server": {
+			{
+				HostID:     host.ID,
+				DaemonID:   daemon.ID,
+				DataSource: HostDataSourceAPI,
+				NextServer: "foo",
+			},
+			{
+				HostID:     host.ID,
+				DaemonID:   daemon.ID,
+				DataSource: HostDataSourceConfig,
+				NextServer: "bar",
+			},
+		},
+		"Server hostname": {
+			{
+				HostID:         host.ID,
+				DaemonID:       daemon.ID,
+				DataSource:     HostDataSourceAPI,
+				ServerHostname: "foo",
+			},
+			{
+				HostID:         host.ID,
+				DaemonID:       daemon.ID,
+				DataSource:     HostDataSourceConfig,
+				ServerHostname: "bar",
+			},
+		},
+	}
+
+	for label, localHosts := range testCases {
+		label := label
+
+		_, _ = DeleteDaemonsFromHost(db, host.ID, HostDataSourceUnspecified)
+		host.LocalHosts = localHosts
+		_ = AddHostLocalHosts(db, &host)
+
+		t.Run(label, func(t *testing.T) {
+			// Act
+			returned, total, err := GetHostsByPage(db, 0, 10, filters, "", SortDirAny)
+
+			// Assert
+			require.NoError(t, err)
+			require.EqualValues(t, total, 1)
+			require.Len(t, returned, 1)
+			require.EqualValues(t, host.ID, returned[0].ID)
+		})
+	}
+}
+
+// Test that page of the hosts contains hosts with conflicted and duplicated
+// local hosts if both conflicted and duplicated DHCP data filter are set.
+func TestGetHostsByPageConflictAndDuplicate(t *testing.T) {
+	// Arrange
+	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	filters := HostsByPageFilters{
+		DHCPDataConflict:  storkutil.Ptr(true),
+		DHCPDataDuplicate: storkutil.Ptr(true),
+	}
+
+	hosts := addTestHosts(t, db)
+	host0 := hosts[0]
+	daemon, _, _ := addTestDaemons(db)
+
+	// Duplicates
+	_ = AddDaemonToHost(db, &host0, daemon.ID, HostDataSourceAPI)
+	_ = AddDaemonToHost(db, &host0, daemon.ID, HostDataSourceConfig)
+
+	// Conflicts
+	host1 := hosts[1]
+	host1.LocalHosts = []LocalHost{
+		{
+			HostID:       host1.ID,
+			DaemonID:     daemon.ID,
+			DataSource:   HostDataSourceAPI,
+			BootFileName: "foo",
+		},
+		{
+			HostID:       host1.ID,
+			DaemonID:     daemon.ID,
+			DataSource:   HostDataSourceConfig,
+			BootFileName: "bar",
+		},
+	}
+	_ = AddHostLocalHosts(db, &host1)
+
+	// Act
+	returned, total, err := GetHostsByPage(db, 0, 10, filters, "", SortDirAny)
+
+	// Assert
+	require.NoError(t, err)
+	require.EqualValues(t, 2, total)
+	require.Len(t, returned, 2)
+	require.EqualValues(t, host0.ID, returned[0].ID)
+	require.EqualValues(t, host1.ID, returned[1].ID)
+}
+
+// Test that the combinations of the conflict and duplicate filters return
+// expected results.
+func TestGetHostsByPageConflictDuplicateCombinations(t *testing.T) {
+	// Arrange
+	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	hosts := addTestHosts(t, db)
+	hostDuplicate := hosts[0]
+	daemon, _, _ := addTestDaemons(db)
+
+	// Duplicates
+	_ = AddDaemonToHost(db, &hostDuplicate, daemon.ID, HostDataSourceAPI)
+	_ = AddDaemonToHost(db, &hostDuplicate, daemon.ID, HostDataSourceConfig)
+
+	// Conflicts
+	hostConflict := hosts[1]
+	hostConflict.LocalHosts = []LocalHost{
+		{
+			HostID:       hostConflict.ID,
+			DaemonID:     daemon.ID,
+			DataSource:   HostDataSourceAPI,
+			BootFileName: "foo",
+		},
+		{
+			HostID:       hostConflict.ID,
+			DaemonID:     daemon.ID,
+			DataSource:   HostDataSourceConfig,
+			BootFileName: "bar",
+		},
+	}
+	_ = AddHostLocalHosts(db, &hostConflict)
+
+	// No conflict and no duplicate.
+	hostStandardIDs := make([]int64, len(hosts)-2)
+	for i, host := range hosts[2:] {
+		hostStandardIDs[i] = host.ID
+	}
+
+	// Filter by conflict -> Return conflicted.
+	t.Run("Conflict", func(t *testing.T) {
+		filters := HostsByPageFilters{
+			DHCPDataConflict:  storkutil.Ptr(true),
+			DHCPDataDuplicate: nil,
+		}
+
+		// Act
+		returned, total, err := GetHostsByPage(db, 0, 10, filters, "", SortDirAny)
+
+		// Assert
+		require.NoError(t, err)
+		require.EqualValues(t, 1, total)
+		require.Len(t, returned, 1)
+		require.EqualValues(t, hostConflict.ID, returned[0].ID)
+	})
+
+	// Filter by duplicate -> Return duplicated.
+	t.Run("Duplicate", func(t *testing.T) {
+		filters := HostsByPageFilters{
+			DHCPDataConflict:  nil,
+			DHCPDataDuplicate: storkutil.Ptr(true),
+		}
+
+		// Act
+		returned, total, err := GetHostsByPage(db, 0, 10, filters, "", SortDirAny)
+
+		// Assert
+		require.NoError(t, err)
+		require.EqualValues(t, 1, total)
+		require.Len(t, returned, 1)
+		require.EqualValues(t, hostDuplicate.ID, returned[0].ID)
+	})
+
+	// Filter by conflict AND duplicate -> Return conflicted and duplicated.
+	t.Run("Conflict and duplicate", func(t *testing.T) {
+		filters := HostsByPageFilters{
+			DHCPDataConflict:  storkutil.Ptr(true),
+			DHCPDataDuplicate: storkutil.Ptr(true),
+		}
+
+		// Act
+		returned, total, err := GetHostsByPage(db, 0, 10, filters, "", SortDirAny)
+
+		// Assert
+		require.NoError(t, err)
+		require.EqualValues(t, 2, total)
+		require.Len(t, returned, 2)
+		require.Contains(t, []int64{hostConflict.ID, hostDuplicate.ID}, returned[0].ID)
+		require.Contains(t, []int64{hostConflict.ID, hostDuplicate.ID}, returned[1].ID)
+	})
+
+	// Filter by NOT conflict -> Return duplicated and single.
+	t.Run("Not conflict", func(t *testing.T) {
+		filters := HostsByPageFilters{
+			DHCPDataConflict:  storkutil.Ptr(false),
+			DHCPDataDuplicate: nil,
+		}
+
+		// Act
+		returned, total, err := GetHostsByPage(db, 0, 10, filters, "", SortDirAny)
+
+		// Assert
+		require.NoError(t, err)
+		require.EqualValues(t, 1+len(hostStandardIDs), total)
+		require.Len(t, returned, 1 + +len(hostStandardIDs))
+		require.Contains(t, append(hostStandardIDs, hostDuplicate.ID), returned[0].ID)
+		require.Contains(t, append(hostStandardIDs, hostDuplicate.ID), returned[1].ID)
+	})
+
+	// Filter by NOT duplicate -> Return conflicted and single.
+	t.Run("Not duplicate", func(t *testing.T) {
+		filters := HostsByPageFilters{
+			DHCPDataConflict:  nil,
+			DHCPDataDuplicate: storkutil.Ptr(false),
+		}
+
+		// Act
+		returned, total, err := GetHostsByPage(db, 0, 10, filters, "", SortDirAny)
+
+		// Assert
+		require.NoError(t, err)
+		require.EqualValues(t, 1 + +len(hostStandardIDs), total)
+		require.Len(t, returned, 1 + +len(hostStandardIDs))
+		require.Contains(t, append(hostStandardIDs, hostConflict.ID), returned[0].ID)
+		require.Contains(t, append(hostStandardIDs, hostConflict.ID), returned[1].ID)
+	})
+
+	// Filter by NOT conflict AND NOT duplicate -> Return single.
+	t.Run("Not conflict and not duplicate", func(t *testing.T) {
+		filters := HostsByPageFilters{
+			DHCPDataConflict:  storkutil.Ptr(false),
+			DHCPDataDuplicate: storkutil.Ptr(false),
+		}
+
+		// Act
+		returned, total, err := GetHostsByPage(db, 0, 10, filters, "", SortDirAny)
+
+		// Assert
+		require.NoError(t, err)
+		require.EqualValues(t, len(hostStandardIDs), total)
+		require.Len(t, returned, len(hostStandardIDs))
+		require.Contains(t, hostStandardIDs, returned[0].ID)
+	})
+
+	// Filter by conflict AND NOT duplicate -> Return conflicted and single.
+	t.Run("Conflict and not duplicate", func(t *testing.T) {
+		filters := HostsByPageFilters{
+			DHCPDataConflict:  storkutil.Ptr(true),
+			DHCPDataDuplicate: storkutil.Ptr(false),
+		}
+
+		// Act
+		returned, total, err := GetHostsByPage(db, 0, 10, filters, "", SortDirAny)
+
+		// Assert
+		require.NoError(t, err)
+		require.EqualValues(t, 1+len(hostStandardIDs), total)
+		require.Len(t, returned, 1+len(hostStandardIDs))
+		require.Contains(t, append(hostStandardIDs, hostConflict.ID), returned[0].ID)
+		require.Contains(t, append(hostStandardIDs, hostConflict.ID), returned[1].ID)
+	})
+
+	// Filter by NOT conflict AND duplicate -> Return duplicated and single.
+	t.Run("Not conflict and duplicate", func(t *testing.T) {
+		filters := HostsByPageFilters{
+			DHCPDataConflict:  storkutil.Ptr(false),
+			DHCPDataDuplicate: storkutil.Ptr(true),
+		}
+
+		// Act
+		returned, total, err := GetHostsByPage(db, 0, 10, filters, "", SortDirAny)
+
+		// Assert
+		require.NoError(t, err)
+		require.EqualValues(t, 1+len(hostStandardIDs), total)
+		require.Len(t, returned, 1+len(hostStandardIDs))
+		require.Contains(t, append(hostStandardIDs, hostDuplicate.ID), returned[0].ID)
+		require.Contains(t, append(hostStandardIDs, hostDuplicate.ID), returned[1].ID)
+	})
+}
+
 // Test hosts can be sorted by different fields.
 func TestGetHostsByPageWithSorting(t *testing.T) {
 	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
@@ -957,13 +1440,13 @@ func TestAddHostLocalHosts(t *testing.T) {
 			"foo",
 			"bar",
 		},
-		DHCPOptionSet: []DHCPOption{
+		DHCPOptionSet: NewDHCPOptionSet([]DHCPOption{
 			{
 				Code:  254,
 				Name:  "foo",
 				Space: dhcpmodel.DHCPv4OptionSpace,
 			},
-		},
+		}, keaconfig.NewHasher()),
 	})
 	err := AddHostLocalHosts(db, &host)
 	require.NoError(t, err)
@@ -1022,10 +1505,10 @@ func TestAddHostLocalHosts(t *testing.T) {
 	require.Equal(t, "bar", returnedList[0].LocalHosts[0].ClientClasses[1])
 
 	// Make sure that DHCP options are returned.
-	require.Len(t, returnedList[0].LocalHosts[0].DHCPOptionSet, 1)
-	require.EqualValues(t, 254, returnedList[0].LocalHosts[0].DHCPOptionSet[0].Code)
-	require.Equal(t, "foo", returnedList[0].LocalHosts[0].DHCPOptionSet[0].Name)
-	require.Equal(t, dhcpmodel.DHCPv4OptionSpace, returnedList[0].LocalHosts[0].DHCPOptionSet[0].Space)
+	require.Len(t, returnedList[0].LocalHosts[0].DHCPOptionSet.Options, 1)
+	require.EqualValues(t, 254, returnedList[0].LocalHosts[0].DHCPOptionSet.Options[0].Code)
+	require.Equal(t, "foo", returnedList[0].LocalHosts[0].DHCPOptionSet.Options[0].Name)
+	require.Equal(t, dhcpmodel.DHCPv4OptionSpace, returnedList[0].LocalHosts[0].DHCPOptionSet.Options[0].Space)
 }
 
 // Test that the host and its local host can be added within a single transaction.
@@ -1675,13 +2158,13 @@ func TestKeaConfigHostInterface(t *testing.T) {
 					"foo",
 					"bar",
 				},
-				DHCPOptionSet: []DHCPOption{
+				DHCPOptionSet: NewDHCPOptionSet([]DHCPOption{
 					{
 						Code:        123,
 						Encapsulate: "dhcp4",
 						Universe:    storkutil.IPv4,
 					},
-				},
+				}, keaconfig.NewHasher()),
 			},
 		},
 	}
@@ -1924,16 +2407,18 @@ func TestSetLocalHost(t *testing.T) {
 	require.EqualValues(t, 234, host.LocalHosts[1].DaemonID)
 	require.Equal(t, HostDataSourceConfig, host.LocalHosts[1].DataSource)
 
-	// Replace the first instance with a new one.
+	// Append a new instance with existing daemon ID but a new data source.
 	host.SetLocalHost(&LocalHost{
 		DaemonID:   123,
 		DataSource: HostDataSourceAPI,
 	})
-	require.Len(t, host.LocalHosts, 2)
+	require.Len(t, host.LocalHosts, 3)
 	require.EqualValues(t, 123, host.LocalHosts[0].DaemonID)
-	require.Equal(t, HostDataSourceAPI, host.LocalHosts[0].DataSource)
+	require.Equal(t, HostDataSourceConfig, host.LocalHosts[0].DataSource)
 	require.EqualValues(t, 234, host.LocalHosts[1].DaemonID)
 	require.Equal(t, HostDataSourceConfig, host.LocalHosts[1].DataSource)
+	require.EqualValues(t, 123, host.LocalHosts[2].DaemonID)
+	require.Equal(t, HostDataSourceAPI, host.LocalHosts[2].DataSource)
 }
 
 // Test that two hosts can be joined by copying LocalHost information from
@@ -2009,3 +2494,268 @@ func TestJoinDifferentHosts(t *testing.T) {
 	ok := host1.Join(host2)
 	require.False(t, ok)
 }
+
+// Test that the daemons are removed from a host properly. Only associations
+// related to the configuration file are removed.
+func TestDeleteDaemonsFromHostConfigDataSource(t *testing.T) {
+	// Arrange
+	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	daemon, _, _ := addTestDaemons(db)
+
+	host := &Host{
+		SubnetID: 0,
+	}
+	_ = AddHost(db, host)
+	_ = AddDaemonToHost(db, host, daemon.ID, HostDataSourceConfig)
+	_ = AddDaemonToHost(db, host, daemon.ID, HostDataSourceAPI)
+
+	host, _ = GetHost(db, host.ID)
+	require.Len(t, host.LocalHosts, 2)
+
+	// Act
+	count, err := DeleteDaemonsFromHost(db, host.ID, HostDataSourceConfig)
+
+	// Assert
+	require.NoError(t, err)
+	require.EqualValues(t, 1, count)
+	host, _ = GetHost(db, host.ID)
+	require.Len(t, host.LocalHosts, 1)
+	require.EqualValues(t, HostDataSourceAPI, host.LocalHosts[0].DataSource)
+}
+
+// Test that the daemons are removed from a host properly. Only associations
+// related to the host database are removed.
+func TestDeleteDaemonsFromHostAPIDataSource(t *testing.T) {
+	// Arrange
+	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	daemon, _, _ := addTestDaemons(db)
+
+	host := &Host{
+		SubnetID: 0,
+	}
+	_ = AddHost(db, host)
+	_ = AddDaemonToHost(db, host, daemon.ID, HostDataSourceConfig)
+	_ = AddDaemonToHost(db, host, daemon.ID, HostDataSourceAPI)
+
+	host, _ = GetHost(db, host.ID)
+	require.Len(t, host.LocalHosts, 2)
+
+	// Act
+	count, err := DeleteDaemonsFromHost(db, host.ID, HostDataSourceAPI)
+
+	// Assert
+	require.NoError(t, err)
+	require.EqualValues(t, 1, count)
+	host, _ = GetHost(db, host.ID)
+	require.Len(t, host.LocalHosts, 1)
+	require.EqualValues(t, HostDataSourceConfig, host.LocalHosts[0].DataSource)
+}
+
+// Test that the daemons are removed from a host properly. All associations
+// are removed.
+func TestDeleteDaemonsFromHostAllDataSource(t *testing.T) {
+	// Arrange
+	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	daemon, _, _ := addTestDaemons(db)
+
+	host := &Host{
+		SubnetID: 0,
+	}
+	_ = AddHost(db, host)
+	_ = AddDaemonToHost(db, host, daemon.ID, HostDataSourceConfig)
+	_ = AddDaemonToHost(db, host, daemon.ID, HostDataSourceAPI)
+
+	host, _ = GetHost(db, host.ID)
+	require.Len(t, host.LocalHosts, 2)
+
+	// Act
+	count, err := DeleteDaemonsFromHost(db, host.ID, HostDataSourceUnspecified)
+
+	// Assert
+	require.NoError(t, err)
+	require.EqualValues(t, 2, count)
+	host, _ = GetHost(db, host.ID)
+	require.Len(t, host.LocalHosts, 0)
+}
+
+// Test that the error is returned if the daemons could not be removed from a
+// host.
+func TestDeleteDaemonsFromHostError(t *testing.T) {
+	// Arrange
+	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	daemon, _, _ := addTestDaemons(db)
+
+	host := &Host{
+		SubnetID: 0,
+	}
+	_ = AddHost(db, host)
+	_ = AddDaemonToHost(db, host, daemon.ID, HostDataSourceConfig)
+	_ = AddDaemonToHost(db, host, daemon.ID, HostDataSourceAPI)
+
+	host, _ = GetHost(db, host.ID)
+	require.Len(t, host.LocalHosts, 2)
+
+	teardown()
+
+	// Act
+	count, err := DeleteDaemonsFromHost(db, host.ID, HostDataSourceUnspecified)
+
+	// Assert
+	require.Error(t, err)
+	require.Zero(t, count)
+}
+
+// Test that the fetching local hosts for a given host ID and config data
+// source works properly.
+func TestGetLocalHostsConfigDataSource(t *testing.T) {
+	// Arrange
+	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	daemon, _, _ := addTestDaemons(db)
+
+	host := &Host{
+		SubnetID: 0,
+	}
+	_ = AddHost(db, host)
+	_ = AddDaemonToHost(db, host, daemon.ID, HostDataSourceConfig)
+	_ = AddDaemonToHost(db, host, daemon.ID, HostDataSourceAPI)
+
+	// Act
+	localHosts, err := GetLocalHosts(db, host.ID, HostDataSourceConfig)
+
+	// Assert
+	require.NoError(t, err)
+	require.Len(t, localHosts, 1)
+	require.EqualValues(t, HostDataSourceConfig, localHosts[0].DataSource)
+}
+
+// Test that the fetching local hosts for a given host ID and API data source
+// works properly.
+func TestGetLocalHostsAPIDataSource(t *testing.T) {
+	// Arrange
+	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	daemon, _, _ := addTestDaemons(db)
+
+	host := &Host{
+		SubnetID: 0,
+	}
+	_ = AddHost(db, host)
+	_ = AddDaemonToHost(db, host, daemon.ID, HostDataSourceConfig)
+	_ = AddDaemonToHost(db, host, daemon.ID, HostDataSourceAPI)
+
+	// Act
+	localHosts, err := GetLocalHosts(db, host.ID, HostDataSourceAPI)
+
+	// Assert
+	require.NoError(t, err)
+	require.Len(t, localHosts, 1)
+	require.EqualValues(t, HostDataSourceAPI, localHosts[0].DataSource)
+}
+
+// Test that the fetching local hosts for a given host ID and all data sources
+// works properly.
+func TestGetLocalHostsAllDataSources(t *testing.T) {
+	// Arrange
+	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	daemon, _, _ := addTestDaemons(db)
+
+	host := &Host{
+		SubnetID: 0,
+	}
+	_ = AddHost(db, host)
+	_ = AddDaemonToHost(db, host, daemon.ID, HostDataSourceConfig)
+	_ = AddDaemonToHost(db, host, daemon.ID, HostDataSourceAPI)
+
+	// Act
+	localHosts, err := GetLocalHosts(db, host.ID, HostDataSourceUnspecified)
+
+	// Assert
+	require.NoError(t, err)
+	require.Len(t, localHosts, 2)
+	dataSources := []HostDataSource{localHosts[0].DataSource, localHosts[1].DataSource}
+	require.Contains(t, dataSources, HostDataSourceConfig)
+	require.Contains(t, dataSources, HostDataSourceAPI)
+}
+
+// Test that the no local hosts and no error are returned for an unknown host
+// ID and any data source works properly.
+func TestGetLocalHostsUnknownHost(t *testing.T) {
+	// Arrange
+	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	daemon, _, _ := addTestDaemons(db)
+
+	host := &Host{
+		SubnetID: 0,
+	}
+	_ = AddHost(db, host)
+	_ = AddDaemonToHost(db, host, daemon.ID, HostDataSourceConfig)
+	_ = AddDaemonToHost(db, host, daemon.ID, HostDataSourceAPI)
+
+	t.Run("config", func(t *testing.T) {
+		// Act
+		localHosts, err := GetLocalHosts(db, 42, HostDataSourceConfig)
+
+		// Assert
+		require.NoError(t, err)
+		require.Len(t, localHosts, 0)
+	})
+
+	t.Run("api", func(t *testing.T) {
+		// Act
+		localHosts, err := GetLocalHosts(db, 42, HostDataSourceAPI)
+
+		// Assert
+		require.NoError(t, err)
+		require.Len(t, localHosts, 0)
+	})
+
+	t.Run("all", func(t *testing.T) {
+		// Act
+		localHosts, err := GetLocalHosts(db, 42, HostDataSourceUnspecified)
+
+		// Assert
+		require.NoError(t, err)
+		require.Len(t, localHosts, 0)
+	})
+}
+
+// Test that the fetching local hosts for an unavailable database causes an
+// error.
+func TestGetLocalHostsError(t *testing.T) {
+	// Arrange
+	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	daemon, _, _ := addTestDaemons(db)
+
+	host := &Host{
+		SubnetID: 0,
+	}
+	_ = AddHost(db, host)
+	_ = AddDaemonToHost(db, host, daemon.ID, HostDataSourceConfig)
+	_ = AddDaemonToHost(db, host, daemon.ID, HostDataSourceAPI)
+
+	teardown()
+
+	// Act
+	localHosts, err := GetLocalHosts(db, host.ID, HostDataSourceAPI)
+
+	// Assert
+	require.Error(t, err)
+	require.Nil(t, localHosts)
+}
