@@ -67,9 +67,9 @@ func (r *RestAPI) convertHostFromRestAPI(dbHost *dbmodel.Host) *models.Host {
 			ServerHostname: dbLocalHost.ServerHostname,
 			BootFileName:   dbLocalHost.BootFileName,
 			ClientClasses:  dbLocalHost.ClientClasses,
-			OptionsHash:    dbLocalHost.DHCPOptionSetHash,
+			OptionsHash:    dbLocalHost.DHCPOptionSet.Hash,
 		}
-		localHost.Options = r.unflattenDHCPOptions(dbLocalHost.DHCPOptionSet, "", 0)
+		localHost.Options = r.unflattenDHCPOptions(dbLocalHost.DHCPOptionSet.Options, "", 0)
 		host.LocalHosts = append(host.LocalHosts, &localHost)
 	}
 	return host
@@ -123,13 +123,11 @@ func (r *RestAPI) convertToHost(restHost *models.Host) (*dbmodel.Host, error) {
 			ServerHostname: lh.ServerHostname,
 			BootFileName:   lh.BootFileName,
 		}
-		localHost.DHCPOptionSet, err = r.flattenDHCPOptions("", lh.Options, 0)
+		options, err := r.flattenDHCPOptions("", lh.Options, 0)
 		if err != nil {
 			return nil, err
 		}
-		if len(localHost.DHCPOptionSet) > 0 {
-			localHost.DHCPOptionSetHash = keaconfig.NewHasher().Hash(localHost.DHCPOptionSet)
-		}
+		localHost.DHCPOptionSet.SetDHCPOptions(options, keaconfig.NewHasher())
 		host.SetLocalHost(&localHost)
 	}
 	return host, nil
@@ -137,15 +135,8 @@ func (r *RestAPI) convertToHost(restHost *models.Host) (*dbmodel.Host, error) {
 
 // Fetches host reservations from the database and converts to the data formats
 // used in REST API.
-func (r *RestAPI) getHosts(offset, limit, appID int64, subnetID *int64, localSubnetID *int64, filterText *string, global *bool, sortField string, sortDir dbmodel.SortDirEnum) (*models.Hosts, error) {
+func (r *RestAPI) getHosts(offset, limit int64, filters dbmodel.HostsByPageFilters, sortField string, sortDir dbmodel.SortDirEnum) (*models.Hosts, error) {
 	// Get the hosts from the database.
-	filters := dbmodel.HostsByPageFilters{
-		AppID:         &appID,
-		SubnetID:      subnetID,
-		LocalSubnetID: localSubnetID,
-		FilterText:    filterText,
-		Global:        global,
-	}
 	dbHosts, total, err := dbmodel.GetHostsByPage(r.DB, offset, limit, filters, sortField, sortDir)
 	if err != nil {
 		return nil, err
@@ -177,13 +168,16 @@ func (r *RestAPI) GetHosts(ctx context.Context, params dhcp.GetHostsParams) midd
 		limit = *params.Limit
 	}
 
-	var appID int64
-	if params.AppID != nil {
-		appID = *params.AppID
+	// Get hosts from DB.
+	filters := dbmodel.HostsByPageFilters{
+		AppID:            params.AppID,
+		SubnetID:         params.SubnetID,
+		LocalSubnetID:    params.LocalSubnetID,
+		FilterText:       params.Text,
+		Global:           params.Global,
+		DHCPDataConflict: params.Conflict,
 	}
-
-	// get hosts from db
-	hosts, err := r.getHosts(start, limit, appID, params.SubnetID, params.LocalSubnetID, params.Text, params.Global, "", dbmodel.SortDirAny)
+	hosts, err := r.getHosts(start, limit, filters, "", dbmodel.SortDirAny)
 	if err != nil {
 		msg := "Problem fetching hosts from the database"
 		log.Error(err)
@@ -235,7 +229,7 @@ func (r *RestAPI) commonCreateOrUpdateHostBegin(ctx context.Context) ([]*models.
 	daemons, err := dbmodel.GetKeaDHCPDaemons(r.DB)
 	if err != nil {
 		msg := "Problem with fetching Kea daemons from the database"
-		log.Error(err)
+		log.WithError(err).Error(msg)
 		return nil, nil, nil, nil, http.StatusInternalServerError, msg
 	}
 	// Convert daemons list to REST API format and extract their configured
@@ -480,7 +474,7 @@ func (r *RestAPI) UpdateHostBegin(ctx context.Context, params dhcp.UpdateHostBeg
 		case errors.As(err, &hostNotFound):
 			// Failed to find host.
 			msg := fmt.Sprintf("Unable to edit the host reservation with ID %d because it cannot be found", params.HostID)
-			log.Error(msg)
+			log.WithError(err).Error("Failed to find host")
 			rsp := dhcp.NewUpdateHostBeginDefault(http.StatusBadRequest).WithPayload(&models.APIError{
 				Message: &msg,
 			})
