@@ -540,7 +540,7 @@ func (r *RestAPI) PingMachine(ctx context.Context, params services.PingMachinePa
 	// ping machine
 	ctx2, cancel := context.WithTimeout(ctx, 10*time.Second)
 	defer cancel()
-	err = r.Agents.Ping(ctx2, dbMachine.Address, dbMachine.AgentPort)
+	err = r.Agents.Ping(ctx2, dbMachine)
 	if err != nil {
 		msg := "Cannot ping machine"
 		log.Error(err)
@@ -919,31 +919,28 @@ func (r *RestAPI) appToRestAPI(dbApp *dbmodel.App) *models.App {
 	isBind9App := dbApp.Type == dbmodel.AppTypeBind9
 
 	agentErrors := int64(0)
-	var agentStats *agentcomm.AgentStats
-	var accessPoint *dbmodel.AccessPoint
+	var agentStats *agentcomm.AgentCommStatsWrapper
 	if dbApp.Machine != nil {
-		agentStats = r.Agents.GetConnectedAgentStats(dbApp.Machine.Address, dbApp.Machine.AgentPort)
+		agentStats = r.Agents.GetConnectedAgentStatsWrapper(dbApp.Machine.Address, dbApp.Machine.AgentPort)
 		if agentStats != nil {
-			agentErrors = agentStats.CurrentErrors
-			accessPoint, _ = dbApp.GetAccessPoint(dbmodel.AccessPointControl)
+			defer agentStats.Close()
+			agentErrors = agentStats.GetStats().GetTotalErrorCount()
 		}
 	}
 
-	if isKeaApp {
-		var keaStats *agentcomm.AgentKeaCommStats
-		if agentStats != nil && accessPoint != nil {
-			keaStats, _ = agentStats.AppCommStats[agentcomm.AppCommStatsKey{
-				Address: accessPoint.Address,
-				Port:    accessPoint.Port,
-			}].(*agentcomm.AgentKeaCommStats)
+	switch {
+	case isKeaApp:
+		var keaStats *agentcomm.KeaAppCommErrorStats
+		if agentStats != nil {
+			keaStats = agentStats.GetStats().GetKeaCommErrorStats(app.ID)
 		}
 		keaDaemons := []*models.KeaDaemon{}
 		for _, d := range dbApp.Daemons {
 			dmn := keaDaemonToRestAPI(d)
 			dmn.AgentCommErrors = agentErrors
 			if keaStats != nil {
-				dmn.CaCommErrors = keaStats.CurrentErrorsCA
-				dmn.DaemonCommErrors = keaStats.CurrentErrorsDaemons[d.Name]
+				dmn.CaCommErrors = keaStats.GetErrorCount(agentcomm.KeaDaemonCA)
+				dmn.DaemonCommErrors = keaStats.GetErrorCount(agentcomm.GetKeaDaemonTypeFromName(d.Name))
 			}
 			keaDaemons = append(keaDaemons, dmn)
 		}
@@ -958,14 +955,14 @@ func (r *RestAPI) appToRestAPI(dbApp *dbmodel.App) *models.App {
 			},
 			models.AppBind9{},
 		}
-	}
-
-	if isBind9App {
+	case isBind9App:
 		var bind9DaemonDB *dbmodel.Daemon
 		var namedStats *dbmodel.Bind9NamedStats
 		if len(dbApp.Daemons) > 0 {
 			bind9DaemonDB = dbApp.Daemons[0]
-			namedStats = bind9DaemonDB.Bind9Daemon.Stats.NamedStats
+			if bind9DaemonDB.Bind9Daemon != nil {
+				namedStats = bind9DaemonDB.Bind9Daemon.Stats.NamedStats
+			}
 		}
 
 		var queryHitRatio float64
@@ -998,24 +995,21 @@ func (r *RestAPI) appToRestAPI(dbApp *dbmodel.App) *models.App {
 				Version:         bind9DaemonDB.Version,
 				Uptime:          bind9DaemonDB.Uptime,
 				ReloadedAt:      convertToOptionalDatetime(bind9DaemonDB.ReloadedAt),
-				ZoneCount:       bind9DaemonDB.Bind9Daemon.Stats.ZoneCount,
-				AutoZoneCount:   bind9DaemonDB.Bind9Daemon.Stats.AutomaticZoneCount,
 				QueryHits:       queryHits,
 				QueryMisses:     queryMisses,
 				QueryHitRatio:   queryHitRatio,
 				AgentCommErrors: agentErrors,
 			}
 		}
+		if bind9DaemonDB.Bind9Daemon != nil {
+			bind9Daemon.ZoneCount = bind9DaemonDB.Bind9Daemon.Stats.ZoneCount
+			bind9Daemon.AutoZoneCount = bind9DaemonDB.Bind9Daemon.Stats.AutomaticZoneCount
+		}
 
-		var bind9Stats *agentcomm.AgentBind9CommStats
-		if agentStats != nil && accessPoint != nil {
-			if bind9Stats, _ = agentStats.AppCommStats[agentcomm.AppCommStatsKey{
-				Address: accessPoint.Address,
-				Port:    accessPoint.Port,
-			}].(*agentcomm.AgentBind9CommStats); bind9Stats != nil {
-				bind9Daemon.RndcCommErrors = bind9Stats.CurrentErrorsRNDC
-				bind9Daemon.StatsCommErrors = bind9Stats.CurrentErrorsStats
-			}
+		if agentStats != nil {
+			bind9Errors := agentStats.GetStats().GetBind9CommErrorStats(app.ID)
+			bind9Daemon.RndcCommErrors = bind9Errors.GetErrorCount(agentcomm.Bind9ChannelRNDC)
+			bind9Daemon.StatsCommErrors = bind9Errors.GetErrorCount(agentcomm.Bind9ChannelStats)
 		}
 		app.Details = struct {
 			models.AppKea
@@ -1073,7 +1067,7 @@ func keaDaemonToRestAPI(dbDaemon *dbmodel.Daemon) *models.KeaDaemon {
 	}
 
 	// Files and backends.
-	if dbDaemon.KeaDaemon.Config != nil {
+	if dbDaemon.KeaDaemon != nil && dbDaemon.KeaDaemon.Config != nil {
 		daemon.Files, daemon.Backends = getKeaStorages(dbDaemon.KeaDaemon.Config.Config)
 	}
 	return daemon
@@ -1162,6 +1156,47 @@ func (r *RestAPI) GetAppsDirectory(ctx context.Context, params services.GetAppsD
 	return rsp
 }
 
+// Returns a list of apps for which the server discovered some communication problems.
+// It includes a lack of communication with the agent or the daemons behind it.
+func (r *RestAPI) GetAppsWithCommunicationIssues(ctx context.Context, params services.GetAppsWithCommunicationIssuesParams) middleware.Responder {
+	// Get all apps with a minimal set of relations.
+	dbApps, err := dbmodel.GetAllAppsWithRelations(r.DB, dbmodel.AppRelationMachine, dbmodel.AppRelationAccessPoints, dbmodel.AppRelationDaemons)
+	if err != nil {
+		msg := "Cannot get apps from the database"
+		log.WithError(err).Error(msg)
+		rsp := services.NewGetAppsDefault(http.StatusInternalServerError).WithPayload(&models.APIError{
+			Message: &msg,
+		})
+		return rsp
+	}
+	apps := []*models.App{}
+	for i := range dbApps {
+		// Convert the apps to the REST API format.
+		app := r.appToRestAPI(&dbApps[i])
+		// Is it a BIND9 daemon?
+		daemon := app.Details.Daemon
+		// Append the app to the list if there is any kind of communication issue.
+		if daemon != nil && daemon.Monitored && (daemon.AgentCommErrors > 0 || daemon.RndcCommErrors > 0 || daemon.StatsCommErrors > 0) {
+			apps = append(apps, app)
+			continue
+		}
+		// Apparently these are Kea daemons.
+		for _, daemon := range app.Details.Daemons {
+			// Append the app to the list if there is any kind of communication issue.
+			if daemon.Monitored && (daemon.AgentCommErrors > 0 || daemon.CaCommErrors > 0 || daemon.DaemonCommErrors > 0) {
+				apps = append(apps, app)
+				break
+			}
+		}
+	}
+	// Send the list.
+	rsp := services.NewGetAppsWithCommunicationIssuesOK().WithPayload(&models.Apps{
+		Items: apps,
+		Total: int64(len(apps)),
+	})
+	return rsp
+}
+
 // Returns an application for a given ID or HTTP 404 status if it's missing.
 func (r *RestAPI) GetApp(ctx context.Context, params services.GetAppParams) middleware.Responder {
 	dbApp, err := dbmodel.GetAppByID(r.DB, params.ID)
@@ -1269,6 +1304,7 @@ func getKeaServicesStatus(db *dbops.PgDB, app *dbmodel.App) *models.ServicesStat
 			}
 		}
 		keaStatus.HaServers = &models.KeaStatusHaServers{
+			Relationship: ha.Relationship,
 			PrimaryServer: &models.KeaHAServerStatus{
 				Age:                age[0],
 				AppID:              appID[0],
@@ -1503,33 +1539,34 @@ func (r *RestAPI) GetDhcpOverview(ctx context.Context, params dhcp.GetDhcpOvervi
 			// will be 0 or 1. Therefore, we take the first HA state if it exists
 			// and return it over the REST API.
 			var (
-				haEnabled   bool
-				haState     string
-				haFailureAt *strfmt.DateTime
+				haEnabled               bool
+				haRelationshipOverviews []*models.DhcpDaemonHARelationshipOverview
+				haState                 string
+				haFailureAt             *strfmt.DateTime
 			)
-			if haOverview := dbDaemon.GetHAOverview(); len(haOverview) > 0 {
+			if overview := dbDaemon.GetHAOverview(); len(overview) > 0 {
 				haEnabled = true
-				haState = haOverview[0].State
-				if !haOverview[0].LastFailureAt.IsZero() {
-					haFailureAt = convertToOptionalDatetime(haOverview[0].LastFailureAt)
+				for i := range overview {
+					haState = overview[i].State
+					if !overview[0].LastFailureAt.IsZero() {
+						haFailureAt = convertToOptionalDatetime(overview[0].LastFailureAt)
+					}
+					haRelationshipOverviews = append(haRelationshipOverviews, &models.DhcpDaemonHARelationshipOverview{
+						HaState:     haState,
+						HaFailureAt: haFailureAt,
+					})
 				}
 			}
 			agentErrors := int64(0)
 			caErrors := int64(0)
 			daemonErrors := int64(0)
-			agentStats := r.Agents.GetConnectedAgentStats(dbApp.Machine.Address, dbApp.Machine.AgentPort)
+			agentStats := r.Agents.GetConnectedAgentStatsWrapper(dbApp.Machine.Address, dbApp.Machine.AgentPort)
 			if agentStats != nil {
-				agentErrors = agentStats.CurrentErrors
-				accessPoint, _ := dbApp.GetAccessPoint(dbmodel.AccessPointControl)
-				if accessPoint != nil {
-					if keaStats, ok := agentStats.AppCommStats[agentcomm.AppCommStatsKey{
-						Address: accessPoint.Address,
-						Port:    accessPoint.Port,
-					}].(*agentcomm.AgentKeaCommStats); ok {
-						caErrors = keaStats.CurrentErrorsCA
-						daemonErrors = keaStats.CurrentErrorsDaemons[dbDaemon.Name]
-					}
-				}
+				defer agentStats.Close()
+				agentErrors = agentStats.GetStats().GetTotalErrorCount()
+				keaErrors := agentStats.GetStats().GetKeaCommErrorStats(dbApp.ID)
+				caErrors = keaErrors.GetErrorCount(agentcomm.KeaDaemonCA)
+				daemonErrors = keaErrors.GetErrorCount(agentcomm.GetKeaDaemonTypeFromName(dbDaemon.Name))
 			}
 			daemon := &models.DhcpDaemon{
 				MachineID:        dbApp.MachineID,
@@ -1543,8 +1580,7 @@ func (r *RestAPI) GetDhcpOverview(ctx context.Context, params dhcp.GetDhcpOvervi
 				Rps1:             int64(dbDaemon.KeaDaemon.KeaDHCPDaemon.Stats.RPS1),
 				Rps2:             int64(dbDaemon.KeaDaemon.KeaDHCPDaemon.Stats.RPS2),
 				HaEnabled:        haEnabled,
-				HaState:          haState,
-				HaFailureAt:      haFailureAt,
+				HaOverview:       haRelationshipOverviews,
 				Uptime:           dbDaemon.Uptime,
 				AgentCommErrors:  agentErrors,
 				CaCommErrors:     caErrors,
