@@ -12,6 +12,7 @@ import (
 	"time"
 
 	"github.com/stretchr/testify/require"
+	gomock "go.uber.org/mock/gomock"
 	keaconfig "isc.org/stork/appcfg/kea"
 	keactrl "isc.org/stork/appctrl/kea"
 	"isc.org/stork/pki"
@@ -31,6 +32,19 @@ import (
 	storkutil "isc.org/stork/util"
 )
 
+//go:generate mockgen -package=restservice -destination=connectedagentsmock_test.go isc.org/stork/server/agentcomm ConnectedAgents
+
+// Type of the function returning agent communication stats wrapped.
+type wrapperFunc = func(string, int64) *agentcomm.AgentCommStatsWrapper
+
+// Convenience function wrapping statistics in AgentCommStatsWrapper and
+// returning in tests mocks.
+func wrap(stats *agentcomm.AgentCommStats) wrapperFunc {
+	return func(string, int64) *agentcomm.AgentCommStatsWrapper {
+		return agentcomm.NewAgentCommStatsWrapper(stats)
+	}
+}
+
 func TestGetVersion(t *testing.T) {
 	db, dbSettings, teardown := dbtest.SetupDatabaseTestCase(t)
 	defer teardown()
@@ -1030,11 +1044,13 @@ func TestRestGetApps(t *testing.T) {
 	db, dbSettings, teardown := dbtest.SetupDatabaseTestCase(t)
 	defer teardown()
 
+	controller := gomock.NewController(t)
+	mock := NewMockConnectedAgents(controller)
+
 	settings := RestAPISettings{}
-	fa := agentcommtest.NewFakeAgents(nil, nil)
 	fec := &storktest.FakeEventCenter{}
 	fd := &storktest.FakeDispatcher{}
-	rapi, err := NewRestAPI(&settings, dbSettings, db, fa, fec, fd)
+	rapi, err := NewRestAPI(&settings, dbSettings, db, mock, fec, fd)
 	require.NoError(t, err)
 	ctx := context.Background()
 
@@ -1099,6 +1115,14 @@ func TestRestGetApps(t *testing.T) {
 	_, err = dbmodel.AddApp(db, s2)
 	require.NoError(t, err)
 
+	stats := agentcomm.NewAgentStats()
+	stats.IncreaseErrorCount("foo")
+	stats.GetKeaCommErrorStats(s1.ID).IncreaseErrorCountBy(agentcomm.KeaDaemonCA, 2)
+	stats.GetKeaCommErrorStats(s1.ID).IncreaseErrorCountBy(agentcomm.KeaDaemonDHCPv4, 5)
+	stats.GetBind9CommErrorStats(s2.ID).IncreaseErrorCountBy(agentcomm.Bind9ChannelRNDC, 2)
+	stats.GetBind9CommErrorStats(s2.ID).IncreaseErrorCountBy(agentcomm.Bind9ChannelStats, 3)
+	mock.EXPECT().GetConnectedAgentStatsWrapper(gomock.Any(), gomock.Any()).DoAndReturn(wrap(stats)).AnyTimes()
+
 	// get added apps
 	params = services.GetAppsParams{}
 	rsp = rapi.GetApps(ctx, params)
@@ -1106,8 +1130,7 @@ func TestRestGetApps(t *testing.T) {
 	okRsp = rsp.(*services.GetAppsOK)
 	require.EqualValues(t, 2, okRsp.Payload.Total)
 
-	// Verify that the communication error counters are returned. See fake_agents.go
-	// to see where those counters are set.
+	// Verify that the communication error counters are returned.
 	require.Len(t, okRsp.Payload.Items, 2)
 	for _, app := range okRsp.Payload.Items {
 		if app.Type == dbmodel.AppTypeKea.String() {
@@ -1200,7 +1223,282 @@ func TestGetAppsDirectory(t *testing.T) {
 	require.Equal(t, bind9App.Name, apps.Items[1].Name)
 }
 
-// Test that status of two HA services for a Kea application is parsed
+// Test that a list of apps with communication issues is returned.
+func TestGetAppsCommunicationIssues(t *testing.T) {
+	db, dbSettings, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	settings := RestAPISettings{}
+	fec := &storktest.FakeEventCenter{}
+	fd := &storktest.FakeDispatcher{}
+	ctx := context.Background()
+
+	// Add a first machine.
+	m1 := &dbmodel.Machine{
+		Address:   "localhost",
+		AgentPort: 8080,
+	}
+	err := dbmodel.AddMachine(db, m1)
+	require.NoError(t, err)
+
+	// Add a second machine.
+	m2 := &dbmodel.Machine{
+		Address:   "localhost",
+		AgentPort: 8081,
+	}
+	err = dbmodel.AddMachine(db, m2)
+	require.NoError(t, err)
+
+	// Add a third machine.
+	m3 := &dbmodel.Machine{
+		Address:   "localhost",
+		AgentPort: 8082,
+	}
+	err = dbmodel.AddMachine(db, m3)
+	require.NoError(t, err)
+
+	// Add Kea app to the first machine.
+	var accessPoints []*dbmodel.AccessPoint
+	accessPoints = dbmodel.AppendAccessPoint(accessPoints, dbmodel.AccessPointControl, "", "", 1234, false)
+	keaApp := &dbmodel.App{
+		MachineID:    m1.ID,
+		Type:         dbmodel.AppTypeKea,
+		Active:       true,
+		Name:         "kea1",
+		AccessPoints: accessPoints,
+		Daemons: []*dbmodel.Daemon{
+			{
+				Name:      "dhcp4",
+				Monitored: true,
+				KeaDaemon: &dbmodel.KeaDaemon{},
+			},
+		},
+	}
+	_, err = dbmodel.AddApp(db, keaApp)
+	require.NoError(t, err)
+	id1 := keaApp.ID
+
+	// Add Kea app to the second machine.
+	accessPoints = []*dbmodel.AccessPoint{}
+	accessPoints = dbmodel.AppendAccessPoint(accessPoints, dbmodel.AccessPointControl, "", "", 2345, false)
+	keaApp = &dbmodel.App{
+		MachineID:    m2.ID,
+		Type:         dbmodel.AppTypeKea,
+		Active:       true,
+		Name:         "kea2",
+		AccessPoints: accessPoints,
+		Daemons: []*dbmodel.Daemon{
+			{
+				Name:      "dhcp4",
+				Monitored: true,
+				KeaDaemon: &dbmodel.KeaDaemon{},
+			},
+		},
+	}
+
+	_, err = dbmodel.AddApp(db, keaApp)
+	require.NoError(t, err)
+
+	// Add Bind9 app to the third machine.
+	accessPoints = []*dbmodel.AccessPoint{}
+	accessPoints = dbmodel.AppendAccessPoint(accessPoints, dbmodel.AccessPointControl, "", "", 3456, false)
+	bind9App := &dbmodel.App{
+		MachineID:    m3.ID,
+		Type:         dbmodel.AppTypeBind9,
+		Active:       true,
+		Name:         "bind9",
+		AccessPoints: accessPoints,
+		Daemons: []*dbmodel.Daemon{
+			{
+				Name:        "bind9",
+				Monitored:   true,
+				Bind9Daemon: &dbmodel.Bind9Daemon{},
+			},
+		},
+	}
+
+	_, err = dbmodel.AddApp(db, bind9App)
+	require.NoError(t, err)
+	id3 := bind9App.ID
+
+	controller := gomock.NewController(t)
+	mock := NewMockConnectedAgents(controller)
+
+	rapi, err := NewRestAPI(&settings, dbSettings, db, mock, fec, fd)
+	require.NoError(t, err)
+
+	t.Run("current errors", func(t *testing.T) {
+		stats1 := agentcomm.NewAgentStats()
+		stats1.IncreaseErrorCount("foo")
+		mock.EXPECT().GetConnectedAgentStatsWrapper(gomock.Any(), int64(8080)).DoAndReturn(wrap(stats1))
+
+		stats2 := agentcomm.NewAgentStats()
+		mock.EXPECT().GetConnectedAgentStatsWrapper(gomock.Any(), int64(8081)).DoAndReturn(wrap(stats2))
+
+		stats3 := agentcomm.NewAgentStats()
+		mock.EXPECT().GetConnectedAgentStatsWrapper(gomock.Any(), int64(8082)).DoAndReturn(wrap(stats3))
+
+		params := services.GetAppsWithCommunicationIssuesParams{}
+		rsp := rapi.GetAppsWithCommunicationIssues(ctx, params)
+		require.IsType(t, &services.GetAppsWithCommunicationIssuesOK{}, rsp)
+		apps := rsp.(*services.GetAppsWithCommunicationIssuesOK).Payload
+		require.EqualValues(t, 1, apps.Total)
+		require.Equal(t, "kea1", apps.Items[0].Name)
+	})
+
+	t.Run("ca errors", func(t *testing.T) {
+		stats1 := agentcomm.NewAgentStats()
+		stats1.GetKeaCommErrorStats(id1).IncreaseErrorCountBy(agentcomm.KeaDaemonCA, 10)
+		mock.EXPECT().GetConnectedAgentStatsWrapper(gomock.Any(), int64(8080)).DoAndReturn(wrap(stats1))
+
+		stats2 := agentcomm.NewAgentStats()
+		mock.EXPECT().GetConnectedAgentStatsWrapper(gomock.Any(), int64(8081)).DoAndReturn(wrap(stats2))
+
+		stats3 := agentcomm.NewAgentStats()
+		mock.EXPECT().GetConnectedAgentStatsWrapper(gomock.Any(), int64(8082)).DoAndReturn(wrap(stats3))
+
+		params := services.GetAppsWithCommunicationIssuesParams{}
+		rsp := rapi.GetAppsWithCommunicationIssues(ctx, params)
+		require.IsType(t, &services.GetAppsWithCommunicationIssuesOK{}, rsp)
+		apps := rsp.(*services.GetAppsWithCommunicationIssuesOK).Payload
+		require.EqualValues(t, 1, apps.Total)
+		require.Equal(t, "kea1", apps.Items[0].Name)
+	})
+
+	t.Run("kea daemon errors", func(t *testing.T) {
+		stats1 := agentcomm.NewAgentStats()
+		stats1.GetKeaCommErrorStats(id1).IncreaseErrorCountBy(agentcomm.KeaDaemonDHCPv4, 1)
+		mock.EXPECT().GetConnectedAgentStatsWrapper(gomock.Any(), int64(8080)).DoAndReturn(wrap(stats1))
+
+		stats2 := agentcomm.NewAgentStats()
+		mock.EXPECT().GetConnectedAgentStatsWrapper(gomock.Any(), int64(8081)).DoAndReturn(wrap(stats2))
+
+		stats3 := agentcomm.NewAgentStats()
+		mock.EXPECT().GetConnectedAgentStatsWrapper(gomock.Any(), int64(8082)).DoAndReturn(wrap(stats3))
+
+		params := services.GetAppsWithCommunicationIssuesParams{}
+		rsp := rapi.GetAppsWithCommunicationIssues(ctx, params)
+		require.IsType(t, &services.GetAppsWithCommunicationIssuesOK{}, rsp)
+		apps := rsp.(*services.GetAppsWithCommunicationIssuesOK).Payload
+		require.EqualValues(t, 1, apps.Total)
+		require.Equal(t, "kea1", apps.Items[0].Name)
+	})
+
+	t.Run("bind9 current errors", func(t *testing.T) {
+		stats1 := agentcomm.NewAgentStats()
+		mock.EXPECT().GetConnectedAgentStatsWrapper(gomock.Any(), int64(8080)).DoAndReturn(wrap(stats1))
+
+		stats2 := agentcomm.NewAgentStats()
+		mock.EXPECT().GetConnectedAgentStatsWrapper(gomock.Any(), int64(8081)).DoAndReturn(wrap(stats2))
+
+		stats3 := agentcomm.NewAgentStats()
+		stats3.IncreaseErrorCount("foo")
+		mock.EXPECT().GetConnectedAgentStatsWrapper(gomock.Any(), int64(8082)).DoAndReturn(wrap(stats3))
+
+		params := services.GetAppsWithCommunicationIssuesParams{}
+		rsp := rapi.GetAppsWithCommunicationIssues(ctx, params)
+		require.IsType(t, &services.GetAppsWithCommunicationIssuesOK{}, rsp)
+		apps := rsp.(*services.GetAppsWithCommunicationIssuesOK).Payload
+		require.EqualValues(t, 1, apps.Total)
+		require.Equal(t, "bind9", apps.Items[0].Name)
+	})
+
+	t.Run("rndc errors", func(t *testing.T) {
+		stats1 := agentcomm.NewAgentStats()
+		mock.EXPECT().GetConnectedAgentStatsWrapper(gomock.Any(), int64(8080)).DoAndReturn(wrap(stats1))
+
+		stats2 := agentcomm.NewAgentStats()
+		mock.EXPECT().GetConnectedAgentStatsWrapper(gomock.Any(), int64(8081)).DoAndReturn(wrap(stats2))
+
+		stats3 := agentcomm.NewAgentStats()
+		stats3.GetBind9CommErrorStats(id3).IncreaseErrorCountBy(agentcomm.Bind9ChannelRNDC, 10)
+		mock.EXPECT().GetConnectedAgentStatsWrapper(gomock.Any(), int64(8082)).DoAndReturn(wrap(stats3))
+
+		params := services.GetAppsWithCommunicationIssuesParams{}
+		rsp := rapi.GetAppsWithCommunicationIssues(ctx, params)
+		require.IsType(t, &services.GetAppsWithCommunicationIssuesOK{}, rsp)
+		apps := rsp.(*services.GetAppsWithCommunicationIssuesOK).Payload
+		require.EqualValues(t, 1, apps.Total)
+		require.Equal(t, "bind9", apps.Items[0].Name)
+	})
+
+	t.Run("stats errors", func(t *testing.T) {
+		stats1 := agentcomm.NewAgentStats()
+		mock.EXPECT().GetConnectedAgentStatsWrapper(gomock.Any(), int64(8080)).DoAndReturn(wrap(stats1))
+
+		stats2 := agentcomm.NewAgentStats()
+		mock.EXPECT().GetConnectedAgentStatsWrapper(gomock.Any(), int64(8081)).DoAndReturn(wrap(stats2))
+
+		stats3 := agentcomm.NewAgentStats()
+		stats3.GetBind9CommErrorStats(id3).IncreaseErrorCountBy(agentcomm.Bind9ChannelStats, 10)
+		mock.EXPECT().GetConnectedAgentStatsWrapper(gomock.Any(), int64(8082)).DoAndReturn(wrap(stats3))
+
+		params := services.GetAppsWithCommunicationIssuesParams{}
+		rsp := rapi.GetAppsWithCommunicationIssues(ctx, params)
+		require.IsType(t, &services.GetAppsWithCommunicationIssuesOK{}, rsp)
+		apps := rsp.(*services.GetAppsWithCommunicationIssuesOK).Payload
+		require.EqualValues(t, 1, apps.Total)
+		require.Equal(t, "bind9", apps.Items[0].Name)
+	})
+}
+
+// Test that non-monitored apps are not returned even when they
+// report communication issues.
+func TestGetAppsCommunicationIssuesNotMonitored(t *testing.T) {
+	db, dbSettings, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	settings := RestAPISettings{}
+	fec := &storktest.FakeEventCenter{}
+	fd := &storktest.FakeDispatcher{}
+	ctx := context.Background()
+
+	// Add a machine.
+	m := &dbmodel.Machine{
+		Address:   "localhost",
+		AgentPort: 8080,
+	}
+	err := dbmodel.AddMachine(db, m)
+	require.NoError(t, err)
+
+	// Add Kea app to the machine.
+	var accessPoints []*dbmodel.AccessPoint
+	accessPoints = dbmodel.AppendAccessPoint(accessPoints, dbmodel.AccessPointControl, "", "", 1234, false)
+	keaApp := &dbmodel.App{
+		MachineID:    m.ID,
+		Type:         dbmodel.AppTypeKea,
+		Active:       true,
+		Name:         "kea1",
+		AccessPoints: accessPoints,
+		Daemons: []*dbmodel.Daemon{
+			{
+				Name:      "dhcp4",
+				Monitored: false,
+				KeaDaemon: &dbmodel.KeaDaemon{},
+			},
+		},
+	}
+	_, err = dbmodel.AddApp(db, keaApp)
+	require.NoError(t, err)
+
+	controller := gomock.NewController(t)
+	mock := NewMockConnectedAgents(controller)
+
+	rapi, err := NewRestAPI(&settings, dbSettings, db, mock, fec, fd)
+	require.NoError(t, err)
+
+	stats := agentcomm.NewAgentStats()
+	stats.IncreaseErrorCount("foo")
+	mock.EXPECT().GetConnectedAgentStatsWrapper(gomock.Any(), int64(8080)).DoAndReturn(wrap(stats))
+
+	params := services.GetAppsWithCommunicationIssuesParams{}
+	rsp := rapi.GetAppsWithCommunicationIssues(ctx, params)
+	require.IsType(t, &services.GetAppsWithCommunicationIssuesOK{}, rsp)
+	apps := rsp.(*services.GetAppsWithCommunicationIssuesOK).Payload
+	require.EqualValues(t, 0, apps.Total)
+}
+
+// Test that status of three HA services for a Kea application is parsed
 // correctly.
 func TestRestGetAppServicesStatus(t *testing.T) {
 	db, dbSettings, teardown := dbtest.SetupDatabaseTestCase(t)
@@ -1251,6 +1549,7 @@ func TestRestGetAppServicesStatus(t *testing.T) {
 			HAService: &dbmodel.BaseHAService{
 				HAType:                      "dhcp4",
 				HAMode:                      "load-balancing",
+				Relationship:                "server1",
 				PrimaryID:                   keaApp.ID,
 				PrimaryStatusCollectedAt:    exampleTime,
 				SecondaryStatusCollectedAt:  exampleTime,
@@ -1272,6 +1571,35 @@ func TestRestGetAppServicesStatus(t *testing.T) {
 				SecondaryAnalyzedPackets:    11,
 			},
 		},
+		{
+			BaseService: dbmodel.BaseService{
+				ServiceType: "ha_dhcp",
+			},
+			HAService: &dbmodel.BaseHAService{
+				HAType:                      "dhcp4",
+				HAMode:                      "load-balancing",
+				Relationship:                "server3",
+				PrimaryID:                   keaApp.ID,
+				PrimaryStatusCollectedAt:    exampleTime,
+				SecondaryStatusCollectedAt:  exampleTime,
+				PrimaryLastState:            "load-balancing",
+				SecondaryLastState:          "load-balancing",
+				PrimaryLastScopes:           []string{"server3"},
+				SecondaryLastScopes:         []string{"server4"},
+				PrimaryLastFailoverAt:       exampleTime,
+				SecondaryLastFailoverAt:     exampleTime,
+				PrimaryCommInterrupted:      &commInterrupted[0],
+				SecondaryCommInterrupted:    &commInterrupted[1],
+				PrimaryConnectingClients:    1,
+				SecondaryConnectingClients:  2,
+				PrimaryUnackedClients:       3,
+				SecondaryUnackedClients:     4,
+				PrimaryUnackedClientsLeft:   5,
+				SecondaryUnackedClientsLeft: 6,
+				PrimaryAnalyzedPackets:      7,
+				SecondaryAnalyzedPackets:    8,
+			},
+		},
 		{
 			BaseService: dbmodel.BaseService{
 				ServiceType: "ha_dhcp",
@@ -1279,6 +1607,7 @@ func TestRestGetAppServicesStatus(t *testing.T) {
 			HAService: &dbmodel.BaseHAService{
 				HAType:                      "dhcp6",
 				HAMode:                      "hot-standby",
+				Relationship:                "server1",
 				PrimaryID:                   keaApp.ID,
 				PrimaryStatusCollectedAt:    exampleTime,
 				SecondaryStatusCollectedAt:  exampleTime,
@@ -1320,13 +1649,14 @@ func TestRestGetAppServicesStatus(t *testing.T) {
 	okRsp := rsp.(*services.GetAppServicesStatusOK)
 	require.NotNil(t, okRsp.Payload.Items)
 
-	// There should be two structures returned, one with a status of
-	// the DHCPv4 server and one with the status of the DHCPv6 server.
-	require.Len(t, okRsp.Payload.Items, 2)
+	// There should be three structures returned, two with the status of
+	// the DHCPv4 server relationships and one with the status of the DHCPv6
+	// server relationship.
+	require.Len(t, okRsp.Payload.Items, 3)
 
 	statusList := okRsp.Payload.Items
 
-	// Validate the status of the DHCPv4 pair.
+	// Validate the status of the first relationship.
 	status := statusList[0].Status.KeaStatus
 	require.NotNil(t, status.HaServers)
 
@@ -1334,6 +1664,8 @@ func TestRestGetAppServicesStatus(t *testing.T) {
 	require.NotNil(t, haStatus.PrimaryServer)
 	require.NotNil(t, haStatus.SecondaryServer)
 
+	require.Equal(t, "server1", haStatus.Relationship)
+
 	require.EqualValues(t, keaApp.Daemons[0].ID, haStatus.PrimaryServer.ID)
 	require.Equal(t, "primary", haStatus.PrimaryServer.Role)
 	require.Len(t, haStatus.PrimaryServer.Scopes, 1)
@@ -1363,7 +1695,7 @@ func TestRestGetAppServicesStatus(t *testing.T) {
 	require.EqualValues(t, 9, haStatus.SecondaryServer.UnackedClientsLeft)
 	require.EqualValues(t, 11, haStatus.SecondaryServer.AnalyzedPackets)
 
-	// Validate the status of the DHCPv6 pair.
+	// Validate the status of the second relationship.
 	status = statusList[1].Status.KeaStatus
 	require.NotNil(t, status.HaServers)
 
@@ -1371,6 +1703,47 @@ func TestRestGetAppServicesStatus(t *testing.T) {
 	require.NotNil(t, haStatus.PrimaryServer)
 	require.NotNil(t, haStatus.SecondaryServer)
 
+	require.Equal(t, "server3", haStatus.Relationship)
+
+	require.EqualValues(t, keaApp.Daemons[0].ID, haStatus.PrimaryServer.ID)
+	require.Equal(t, "primary", haStatus.PrimaryServer.Role)
+	require.Len(t, haStatus.PrimaryServer.Scopes, 1)
+	require.Contains(t, haStatus.PrimaryServer.Scopes, "server3")
+	require.Equal(t, "load-balancing", haStatus.PrimaryServer.State)
+	require.GreaterOrEqual(t, haStatus.PrimaryServer.Age, int64(5))
+	require.Equal(t, "127.0.0.1", haStatus.PrimaryServer.ControlAddress)
+	require.EqualValues(t, keaApp.ID, haStatus.PrimaryServer.AppID)
+	require.NotEmpty(t, haStatus.PrimaryServer.StatusTime.String())
+	require.EqualValues(t, 1, haStatus.PrimaryServer.CommInterrupted)
+	require.EqualValues(t, 1, haStatus.PrimaryServer.ConnectingClients)
+	require.EqualValues(t, 3, haStatus.PrimaryServer.UnackedClients)
+	require.EqualValues(t, 5, haStatus.PrimaryServer.UnackedClientsLeft)
+	require.EqualValues(t, 7, haStatus.PrimaryServer.AnalyzedPackets)
+
+	require.Equal(t, "secondary", haStatus.SecondaryServer.Role)
+	require.Len(t, haStatus.SecondaryServer.Scopes, 1)
+	require.Contains(t, haStatus.SecondaryServer.Scopes, "server4")
+	require.Equal(t, "load-balancing", haStatus.SecondaryServer.State)
+	require.GreaterOrEqual(t, haStatus.SecondaryServer.Age, int64(5))
+	require.False(t, haStatus.SecondaryServer.InTouch)
+	require.Empty(t, haStatus.SecondaryServer.ControlAddress)
+	require.NotEmpty(t, haStatus.SecondaryServer.StatusTime.String())
+	require.EqualValues(t, 1, haStatus.SecondaryServer.CommInterrupted)
+	require.EqualValues(t, 2, haStatus.SecondaryServer.ConnectingClients)
+	require.EqualValues(t, 4, haStatus.SecondaryServer.UnackedClients)
+	require.EqualValues(t, 6, haStatus.SecondaryServer.UnackedClientsLeft)
+	require.EqualValues(t, 8, haStatus.SecondaryServer.AnalyzedPackets)
+
+	// Validate the status of the DHCPv6 pair.
+	status = statusList[2].Status.KeaStatus
+	require.NotNil(t, status.HaServers)
+
+	haStatus = status.HaServers
+	require.NotNil(t, haStatus.PrimaryServer)
+	require.NotNil(t, haStatus.SecondaryServer)
+
+	require.Equal(t, "server1", haStatus.Relationship)
+
 	require.EqualValues(t, keaApp.Daemons[0].ID, haStatus.PrimaryServer.ID)
 	require.Equal(t, "primary", haStatus.PrimaryServer.Role)
 	require.Len(t, haStatus.PrimaryServer.Scopes, 1)
@@ -1447,6 +1820,7 @@ func TestRestGetAppServicesStatusPassiveBackup(t *testing.T) {
 			HAService: &dbmodel.BaseHAService{
 				HAType:                   "dhcp4",
 				HAMode:                   "passive-backup",
+				Relationship:             "server1",
 				PrimaryID:                keaApp.ID,
 				PrimaryStatusCollectedAt: exampleTime,
 				PrimaryLastState:         "passive-backup",
@@ -1574,19 +1948,11 @@ func TestGetDhcpOverview(t *testing.T) {
 	db, dbSettings, teardown := dbtest.SetupDatabaseTestCase(t)
 	defer teardown()
 
-	settings := RestAPISettings{}
-	fa := agentcommtest.NewFakeAgents(nil, nil)
-	fec := &storktest.FakeEventCenter{}
-	fd := &storktest.FakeDispatcher{}
-	rapi, err := NewRestAPI(&settings, dbSettings, db, fa, fec, fd)
-	require.NoError(t, err)
-	ctx := context.Background()
-
 	m := &dbmodel.Machine{
 		Address:   "localhost",
 		AgentPort: 8080,
 	}
-	err = dbmodel.AddMachine(db, m)
+	err := dbmodel.AddMachine(db, m)
 	require.NoError(t, err)
 
 	// add app kea to machine
@@ -1624,6 +1990,22 @@ func TestGetDhcpOverview(t *testing.T) {
 	})
 	require.NoError(t, err)
 
+	controller := gomock.NewController(t)
+	mock := NewMockConnectedAgents(controller)
+	stats := agentcomm.NewAgentStats()
+	stats.IncreaseErrorCount("foo")
+	stats.GetKeaCommErrorStats(app.ID).IncreaseErrorCountBy(agentcomm.KeaDaemonCA, 2)
+	stats.GetKeaCommErrorStats(app.ID).IncreaseErrorCountBy(agentcomm.KeaDaemonDHCPv4, 5)
+	mock.EXPECT().GetConnectedAgentStatsWrapper(gomock.Any(), int64(8080)).DoAndReturn(wrap(stats))
+
+	settings := RestAPISettings{}
+	//	fa := agentcommtest.NewFakeAgents(nil, nil)
+	fec := &storktest.FakeEventCenter{}
+	fd := &storktest.FakeDispatcher{}
+	rapi, err := NewRestAPI(&settings, dbSettings, db, mock, fec, fd)
+	require.NoError(t, err)
+	ctx := context.Background()
+
 	// get overview, generally it should be empty
 	params := dhcp.GetDhcpOverviewParams{}
 	rsp := rapi.GetDhcpOverview(ctx, params)
@@ -1645,7 +2027,7 @@ func TestGetDhcpOverview(t *testing.T) {
 
 	// HA is not enabled.
 	require.False(t, okRsp.Payload.DhcpDaemons[0].HaEnabled)
-	require.Empty(t, okRsp.Payload.DhcpDaemons[0].HaState)
+	require.Empty(t, okRsp.Payload.DhcpDaemons[0].HaOverview)
 }
 
 // This test verifies that the overview response includes HA state.
@@ -1697,6 +2079,7 @@ func TestHAInDhcpOverview(t *testing.T) {
 		HAService: &dbmodel.BaseHAService{
 			HAType:                   "dhcp4",
 			HAMode:                   "load-balancing",
+			Relationship:             "server1",
 			PrimaryID:                keaApp.ID,
 			PrimaryStatusCollectedAt: exampleTime,
 			PrimaryLastState:         "load-balancing",
@@ -1719,8 +2102,9 @@ func TestHAInDhcpOverview(t *testing.T) {
 
 	// Test that the HA specific information was returned.
 	require.True(t, okRsp.Payload.DhcpDaemons[0].HaEnabled)
-	require.Equal(t, "load-balancing", okRsp.Payload.DhcpDaemons[0].HaState)
-	require.NotEmpty(t, okRsp.Payload.DhcpDaemons[0].HaFailureAt.String())
+	require.Len(t, okRsp.Payload.DhcpDaemons[0].HaOverview, 1)
+	require.Equal(t, "load-balancing", okRsp.Payload.DhcpDaemons[0].HaOverview[0].HaState)
+	require.NotEmpty(t, okRsp.Payload.DhcpDaemons[0].HaOverview[0].HaFailureAt.String())
 }
 
 // Test that the DHCP Overview is properly generated when the statistic
