@@ -21,11 +21,11 @@ import (
 func newRestUser(u dbmodel.SystemUser) *models.User {
 	id := int64(u.ID)
 	r := &models.User{
-		Email:                  &u.Email,
-		Name:                   &u.Name,
+		Email:                  u.Email,
+		Name:                   u.Name,
 		ID:                     &id,
-		Lastname:               &u.Lastname,
-		Login:                  &u.Login,
+		Lastname:               u.Lastname,
+		Login:                  u.Login,
 		AuthenticationMethodID: &u.AuthenticationMethodID,
 		ExternalID:             u.ExternalID,
 		Groups:                 []int64{},
@@ -92,13 +92,24 @@ func (r *RestAPI) externalAuthentication(ctx context.Context, params users.Creat
 	)
 
 	if calloutUser == nil || err != nil {
-		return nil, errors.Errorf("cannot authenticate a user")
+		return nil, errors.WithMessage(err, "cannot authenticate a user")
+	}
+
+	groupIDMapping := map[authenticationcallouts.UserGroupID]int{
+		authenticationcallouts.UserGroupIDSuperAdmin: dbmodel.SuperAdminGroupID,
+		authenticationcallouts.UserGroupIDAdmin:      dbmodel.AdminGroupID,
 	}
 
 	var groups []*dbmodel.SystemGroup
 	for _, g := range calloutUser.Groups {
+		systemGroupID, ok := groupIDMapping[g]
+		if !ok {
+			log.Warningf("Unknown group returned from hook: %d", g)
+			continue
+		}
+
 		groups = append(groups, &dbmodel.SystemGroup{
-			ID: g,
+			ID: systemGroupID,
 		})
 	}
 
@@ -160,7 +171,13 @@ func (r *RestAPI) CreateSession(ctx context.Context, params users.CreateSessionP
 		systemUser, err = r.externalAuthentication(ctx, params)
 	}
 
-	if systemUser == nil || err != nil {
+	if systemUser == nil && err == nil {
+		log.
+			WithField("method", authenticationMethod).
+			WithField("identifier", *params.Credentials.Identifier).
+			Error("User not found, cannot authenticate")
+		return users.NewCreateSessionBadRequest()
+	} else if err != nil {
 		log.
 			WithError(err).
 			WithField("method", authenticationMethod).
@@ -289,21 +306,23 @@ func (r *RestAPI) CreateUser(ctx context.Context, params users.CreateUserParams)
 	u := params.Account.User
 	p := params.Account.Password
 
-	if u == nil || u.Login == nil || u.Email == nil || u.Lastname == nil || u.Name == nil || p == nil {
-		log.Warn("Failed to create new user account: missing data")
-
+	if u == nil || p == nil || u.Name == "" || u.Lastname == "" {
 		msg := "Failed to create new user account: missing data"
-		rspErr := models.APIError{
-			Message: &msg,
-		}
+		log.Warn(msg)
+		rspErr := models.APIError{Message: &msg}
+		return users.NewCreateUserDefault(http.StatusBadRequest).WithPayload(&rspErr)
+	} else if u.Login == "" && u.Email == "" {
+		msg := "Failed to create new user account: missing identifier"
+		log.Warn(msg)
+		rspErr := models.APIError{Message: &msg}
 		return users.NewCreateUserDefault(http.StatusBadRequest).WithPayload(&rspErr)
 	}
 
 	su := &dbmodel.SystemUser{
-		Login:    *u.Login,
-		Email:    *u.Email,
-		Lastname: *u.Lastname,
-		Name:     *u.Name,
+		Login:    u.Login,
+		Email:    u.Email,
+		Lastname: u.Lastname,
+		Name:     u.Name,
 	}
 
 	for _, gid := range u.Groups {
@@ -354,10 +373,24 @@ func (r *RestAPI) UpdateUser(ctx context.Context, params users.UpdateUserParams)
 	u := params.Account.User
 	p := params.Account.Password
 
-	if u == nil || u.ID == nil || u.Login == nil || u.Email == nil || u.Lastname == nil || u.Name == nil {
-		log.Warn("Failed to update user account: missing data")
-
+	if u == nil || u.ID == nil {
 		msg := "Failed to update user account: missing data"
+		log.Warn(msg)
+		rspErr := models.APIError{
+			Message: &msg,
+		}
+		return users.NewUpdateUserDefault(http.StatusBadRequest).WithPayload(&rspErr)
+	} else if u.Login == "" && u.Email == "" {
+		msg := "Failed to create new user account: missing identifier"
+		log.Warn(msg)
+		rspErr := models.APIError{Message: &msg}
+		return users.NewCreateUserDefault(http.StatusBadRequest).WithPayload(&rspErr)
+	}
+
+	isInternal := u.AuthenticationMethodID == nil || *u.AuthenticationMethodID == dbmodel.AuthenticationMethodIDInternal
+	if isInternal && (u.Name == "" || u.Lastname == "") {
+		msg := "Failed to update user account: missing first or last name"
+		log.Warn(msg)
 		rspErr := models.APIError{
 			Message: &msg,
 		}
@@ -366,10 +399,10 @@ func (r *RestAPI) UpdateUser(ctx context.Context, params users.UpdateUserParams)
 
 	su := &dbmodel.SystemUser{
 		ID:       int(*u.ID),
-		Login:    *u.Login,
-		Email:    *u.Email,
-		Lastname: *u.Lastname,
-		Name:     *u.Name,
+		Login:    u.Login,
+		Email:    u.Email,
+		Lastname: u.Lastname,
+		Name:     u.Name,
 	}
 
 	for _, gid := range u.Groups {
@@ -389,8 +422,8 @@ func (r *RestAPI) UpdateUser(ctx context.Context, params users.UpdateUserParams)
 	} else if err != nil {
 		log.WithFields(log.Fields{
 			"userID": *u.ID,
-			"login":  *u.Login,
-			"email":  *u.Email,
+			"login":  u.Login,
+			"email":  u.Email,
 		}).WithError(err).Errorf("Failed to update user account for user %s", su.Identity())
 
 		msg := fmt.Sprintf("Failed to update user account for user %s", su.Identity())
@@ -410,8 +443,8 @@ func (r *RestAPI) UpdateUser(ctx context.Context, params users.UpdateUserParams)
 		if err != nil {
 			log.WithFields(log.Fields{
 				"userID": *u.ID,
-				"login":  *u.Login,
-				"email":  *u.Email,
+				"login":  u.Login,
+				"email":  u.Email,
 			}).WithError(err).Errorf("Failed to update password for user %s", su.Identity())
 
 			msg := fmt.Sprintf("Failed to update password for user %s", su.Identity())
