@@ -91,6 +91,27 @@ func TestIsHexIdentifier(t *testing.T) {
 	require.False(t, IsHexIdentifier("12:56:9"))
 	require.False(t, IsHexIdentifier("ab,cd"))
 	require.False(t, IsHexIdentifier("ab: cd"))
+	require.False(t, IsHexIdentifier("abcde"))
+}
+
+// Test splitting an identifier into a slice of bytes.
+func TestCountHexIdentifierBytes(t *testing.T) {
+	require.Equal(t, 3, CountHexIdentifierBytes("01:02:03"))
+	require.Equal(t, 3, CountHexIdentifierBytes("01::02::03"))
+	require.Equal(t, 3, CountHexIdentifierBytes("01 e2 03"))
+	require.Equal(t, 3, CountHexIdentifierBytes("abcdef "))
+	require.Equal(t, 1, CountHexIdentifierBytes("12"))
+	require.Equal(t, 3, CountHexIdentifierBytes(" abcd:ef"))
+
+	// Invalid output for invalid input.
+	require.Zero(t, CountHexIdentifierBytes(" "))
+	require.Zero(t, CountHexIdentifierBytes(""))
+	require.Zero(t, CountHexIdentifierBytes("1234gh"))
+	require.Zero(t, CountHexIdentifierBytes("12:56:"))
+	require.Zero(t, CountHexIdentifierBytes("12:56:9"))
+	require.Zero(t, CountHexIdentifierBytes("ab,cd"))
+	require.Zero(t, CountHexIdentifierBytes("ab: cd"))
+	require.Zero(t, CountHexIdentifierBytes("abcde"))
 }
 
 // Check if BytesToHex works.
