@@ -34,6 +34,7 @@ import { DataViewModule } from 'primeng/dataview'
 import { ToggleButtonModule } from 'primeng/togglebutton'
 import { ProgressSpinnerModule } from 'primeng/progressspinner'
 import { PlaceholderPipe } from '../pipes/placeholder.pipe'
+import { ServerSentEventsService, ServerSentEventsTestingService } from '../server-sent-events.service'
 import { DividerModule } from 'primeng/divider'
 import { TagModule } from 'primeng/tag'
 import { EventTextComponent } from '../event-text/event-text.component'
@@ -107,7 +108,14 @@ describe('KeaAppTabComponent', () => {
 
     beforeEach(waitForAsync(() => {
         TestBed.configureTestingModule({
-            providers: [UsersService, DHCPService, ServicesService, MessageService, MockLocationStrategy],
+            providers: [
+                UsersService,
+                DHCPService,
+                ServicesService,
+                MessageService,
+                MockLocationStrategy,
+                { provide: ServerSentEventsService, useClass: ServerSentEventsTestingService },
+            ],
             imports: [
                 RouterTestingModule,
                 TableModule,
